require 'runit/testcase'
require 'refe/inheritancegraph'

class InheritanceGraphTester < RUNIT::TestCase

  testme!

  def setup
    @graph = ReFe::InheritanceGraph.parse('data/inheritance_graph')
  end

  NODES = File.readlines('data/igraph_node_list').map {|line| line.strip }.sort

  def nodes( g = @graph )
    g.instance_eval { @nodes }.keys.sort
  end

  def test_s_parse
    assert_instance_of ReFe::InheritanceGraph, @graph
    assert_equal NODES, nodes()
  end

  def test_s_new
    g = ReFe::InheritanceGraph.new
    assert_instance_of ReFe::InheritanceGraph, g
    assert_equal ['Object'], nodes(g)
  end

  def test_add_class_node
    @graph.add_class_node 'NewClass', 'Object'
    assert_equal((NODES + %w(NewClass)).sort, nodes())
    @graph.add_class_node 'NewClass2', 'Object'
    assert_equal((NODES + %w(NewClass NewClass2)).sort, nodes())
    assert_exception(ReFe::InvalidGraph) {
      @graph.add_class_node 'NewClass', 'Object'
    }
    assert_exception(ReFe::InvalidGraph) {
      @graph.add_class_node 'NewClass3', 'NotExist'
    }
  end

  def test_add_include_edge
    @graph.add_include_edge 'Array', 'NewModule'
    assert_equal((NODES + %w(NewModule)).sort, nodes())
    @graph.add_include_edge 'Array', 'NewModule2'
    assert_equal((NODES + %w(NewModule NewModule2)).sort, nodes())
    assert_exception(ReFe::InvalidGraph) {
      @graph.add_include_edge 'NotExist', 'NewModule'
    }
    assert_exception(ReFe::InvalidGraph) {
      @graph.add_include_edge 'Array', 'Object'
    }
  end

  def test_ancestors_of
    assert_equal %w(Object), @graph.ancestors_of('Object')
    assert_equal %w(Array Enumerable Object), @graph.ancestors_of('Array')
    assert_equal %w(Enumerable), @graph.ancestors_of('Enumerable')
    assert_equal %w(Comparable), @graph.ancestors_of('Comparable')
    assert_equal %w(TCPSocket IPSocket BasicSocket IO Enumerable File::Constants Object), @graph.ancestors_of('TCPSocket')
    assert_equal %w(Fixnum Integer Precision Numeric Comparable Object), @graph.ancestors_of('Fixnum')
    assert_equal %w(NotExist), @graph.ancestors_of('NotExist')
  end

  def test_superclasses_of
    assert_equal %w(), @graph.superclasses_of('Object')
    assert_equal %w(Object), @graph.superclasses_of('Array')
    assert_equal %w(Numeric Object), @graph.superclasses_of('Integer')
    assert_equal %w(Integer Numeric Object), @graph.superclasses_of('Fixnum')
  end

  def test_true_ancestors_of
    assert_equal %w(), @graph.superclasses_of('Object')
    assert_equal %w(), @graph.superclasses_of('Kernel')
    assert_equal %w(), @graph.superclasses_of('Enumerable')
    assert_equal %w(), @graph.superclasses_of('Comparable')
    assert_equal %w(Object), @graph.superclasses_of('Array')
    assert_equal %w(Object), @graph.superclasses_of('Numeric')
    assert_equal %w(Numeric Object), @graph.superclasses_of('Integer')
    assert_equal %w(Integer Numeric Object), @graph.superclasses_of('Fixnum')
    assert_equal %w(), @graph.superclasses_of('NotExist')
  end

end
